@@ -12,9 +12,6 @@ enum {
 	TIMER_FEATURE = 0,
 	TIMER = (1 << TIMER_FEATURE),
 
-	WRITE_THROUGH_FEATURE = 1,
-	WRITE_THROUGH = (1 << WRITE_THROUGH_FEATURE),
-
 	__CACHE_MAX_FEATURE
 };
 #define CACHE_MAX_FEATURE __CACHE_MAX_FEATURE
@@ -46,9 +43,7 @@ struct cache_feature {
 	int  (*dump)(struct cache_object *obj, void *data, char *buf, int type);
 };
 
-extern struct cache_feature lifetime_feature;
 extern struct cache_feature timer_feature;
-extern struct cache_feature writethrough_feature;
 
 #define CACHE_MAX_NAMELEN 32
 
@@ -121,8 +116,10 @@ void *cache_get_extra(struct cache *, void *);
 void cache_iterate(struct cache *c, void *data, int (*iterate)(void *data1, void *data2));
 
 /* iterators */
+struct nfct_handle;
+
 void cache_dump(struct cache *c, int fd, int type);
-void cache_commit(struct cache *c);
+void cache_commit(struct cache *c, struct nfct_handle *h);
 void cache_flush(struct cache *c);
 void cache_bulk(struct cache *c);
 
