@@ -31,9 +31,11 @@
 #define STATS_CACHE	29	/* extended cache stats			*/
 #define STATS_RUNTIME	30	/* extended runtime stats		*/
 #define STATS_LINK	31	/* dedicated link stats			*/
-#define STATS_QUEUE	32	/* queue stats				*/
+#define STATS_RSQUEUE	32	/* resend queue stats			*/
 #define FLUSH_INT_CACHE	33	/* flush internal cache			*/
 #define FLUSH_EXT_CACHE	34	/* flush external cache			*/
+#define STATS_PROCESS	35	/* child process stats			*/
+#define STATS_QUEUE	36	/* queue stats				*/
 
 #define DEFAULT_CONFIGFILE	"/etc/conntrackd/conntrackd.conf"
 #define DEFAULT_LOCKFILE	"/var/lock/conntrackd.lock"
@@ -91,9 +93,12 @@ struct ct_conf {
 	unsigned int resend_queue_size; /* FTFW protocol */
 	unsigned int window_size;
 	int poll_kernel_secs;
-	int cache_write_through;
 	int filter_from_kernelspace;
 	int event_iterations_limit;
+	struct {
+		int type;
+		int prio;
+	} sched;
 	struct {
 		char logfile[FILENAME_MAXLEN];
 		int syslog_facility;
@@ -116,10 +121,10 @@ struct ct_general_state {
 	int				event_iterations_limit;
 
 	struct nfct_handle		*dump;		/* dump handler */
-	struct nfct_handle		*request;	/* request handler */
 	struct nfct_handle		*resync;	/* resync handler */
 	struct nfct_handle		*get;		/* get handler */
 	int				get_retval;	/* hackish */
+	struct nfct_handle		*flush;		/* flusher */
 
 	struct alarm_block		resync_alarm;
 	struct alarm_block		polling_alarm;
@@ -163,6 +168,8 @@ struct ct_sync_state {
 	struct cache *internal; 	/* internal events cache (netlink) */
 	struct cache *external; 	/* external events cache (mcast) */
 
+	struct nfct_handle *commit;
+
 	struct multichannel	*channel;
 	struct nlif_handle	*interface;
 	struct queue *tx_queue;
@@ -204,12 +211,6 @@ extern struct ct_conf conf;
 extern union ct_state state;
 extern struct ct_general_state st;
 
-#ifndef IPPROTO_VRRP
-#define IPPROTO_VRRP 112
-#endif
-
-#define STEPS_PER_SECONDS	5
-
 struct ct_mode {
 	int (*init)(void);
 	void (*run)(fd_set *readfds);
@@ -220,9 +221,9 @@ struct ct_mode {
 		      struct nf_conntrack *ct,
 		      void *data);
 	int (*purge)(void);
-	void (*event_new)(struct nf_conntrack *ct);
-	void (*event_upd)(struct nf_conntrack *ct);
-	int (*event_dst)(struct nf_conntrack *ct);
+	void (*event_new)(struct nf_conntrack *ct, int origin);
+	void (*event_upd)(struct nf_conntrack *ct, int origin);
+	int (*event_dst)(struct nf_conntrack *ct, int origin);
 };
 
 /* conntrackd modes */
