@@ -18,10 +18,8 @@ struct mcast_conf {
 		struct in_addr interface_addr;
 		unsigned int interface_index6;
 	} ifa;
-	int mtu;
 	int sndbuf;
 	int rcvbuf;
-	char iface[IFNAMSIZ];
 };
 
 struct mcast_stats {
@@ -46,10 +44,16 @@ void mcast_server_destroy(struct mcast_sock *m);
 struct mcast_sock *mcast_client_create(struct mcast_conf *conf);
 void mcast_client_destroy(struct mcast_sock *m);
 
-ssize_t mcast_send(struct mcast_sock *m, void *data, int size);
+ssize_t mcast_send(struct mcast_sock *m, const void *data, int size);
 ssize_t mcast_recv(struct mcast_sock *m, void *data, int size);
 
-struct mcast_stats *mcast_get_stats(struct mcast_sock *m);
-void mcast_dump_stats(int fd, struct mcast_sock *s, struct mcast_sock *r);
+int mcast_get_fd(struct mcast_sock *m);
+
+int mcast_snprintf_stats(char *buf, size_t buflen, char *ifname,
+			 struct mcast_stats *s, struct mcast_stats *r);
+
+int mcast_snprintf_stats2(char *buf, size_t buflen, const char *ifname,
+			  const char *status, int active,
+			  struct mcast_stats *s, struct mcast_stats *r);
 
 #endif
