@@ -95,7 +95,6 @@ static int compare(const void *data1, const void *data2)
 
 struct cache_feature *cache_feature[CACHE_MAX_FEATURE] = {
 	[TIMER_FEATURE]		= &timer_feature,
-	[WRITE_THROUGH_FEATURE] = &writethrough_feature,
 };
 
 struct cache *cache_create(const char *name, 
@@ -341,8 +340,10 @@ cache_update_force(struct cache *c, struct nf_conntrack *ct)
 	if (obj == NULL)
 		return NULL;
 
-	if (cache_add(c, obj, id) == -1)
+	if (cache_add(c, obj, id) == -1) {
+		cache_object_free(obj);
 		return NULL;
+	}
 
 	return obj;
 }
