@@ -57,7 +57,7 @@ static struct parser h[NTA_MAX] = {
 		.attr	= ATTR_L4PROTO,
 		.size	= NTA_SIZE(sizeof(uint8_t)),
 	},
-	[NTA_STATE] = {
+	[NTA_TCP_STATE] = {
 		.parse	= parse_u8,
 		.attr	= ATTR_TCP_STATE,
 		.size	= NTA_SIZE(sizeof(uint8_t)),
@@ -121,6 +121,31 @@ static struct parser h[NTA_MAX] = {
 		.parse	= parse_nat_seq_adj,
 		.size	= NTA_SIZE(sizeof(struct nta_attr_natseqadj)),
 	},
+	[NTA_SCTP_STATE] = {
+		.parse	= parse_u8,
+		.attr	= ATTR_SCTP_STATE,
+		.size	= NTA_SIZE(sizeof(uint8_t)),
+	},
+	[NTA_SCTP_VTAG_ORIG] = {
+		.parse	= parse_u32,
+		.attr	= ATTR_SCTP_VTAG_ORIG,
+		.size	= NTA_SIZE(sizeof(uint32_t)),
+	},
+	[NTA_SCTP_VTAG_REPL] = {
+		.parse	= parse_u32,
+		.attr	= ATTR_SCTP_VTAG_REPL,
+		.size	= NTA_SIZE(sizeof(uint32_t)),
+	},
+	[NTA_DCCP_STATE] = {
+		.parse	= parse_u8,
+		.attr	= ATTR_DCCP_STATE,
+		.size	= NTA_SIZE(sizeof(uint8_t)),
+	},
+	[NTA_DCCP_ROLE] = {
+		.parse	= parse_u8,
+		.attr	= ATTR_DCCP_ROLE,
+		.size	= NTA_SIZE(sizeof(uint8_t)),
+	},
 };
 
 static void
