@@ -26,6 +26,8 @@
 #include "fds.h"
 #include "event.h"
 #include "queue.h"
+#include "process.h"
+#include "origin.h"
 
 #include <errno.h>
 #include <unistd.h>
@@ -205,7 +207,15 @@ static void do_reset_cache_alarm(struct alarm_block *a, void *data)
 {
 	STATE(stats).nl_kernel_table_flush++;
 	dlog(LOG_NOTICE, "flushing kernel conntrack table (scheduled)");
-	nl_flush_conntrack_table(STATE(request));
+
+	/* fork a child process that performs the flush operation,
+	 * meanwhile the parent process handles events. */
+	if (fork_process_new(CTD_PROC_FLUSH, CTD_PROC_F_EXCL,
+			     NULL, NULL) == 0) {
+		nl_flush_conntrack_table(STATE(flush));
+		exit(EXIT_SUCCESS);
+	}
+	/* this is not required if events don't get lost */
 	cache_flush(STATE_SYNC(internal));
 }
 
@@ -246,10 +256,6 @@ static int init_sync(void)
 		return -1;
 	}
 
-	/* straight forward commit of conntrack to kernel space */
-	if (CONFIG(cache_write_through))
-		STATE_SYNC(sync)->external_cache_flags |= WRITE_THROUGH;
-
 	STATE_SYNC(external) = 
 		cache_create("external",
 			     STATE_SYNC(sync)->external_cache_flags,
@@ -283,7 +289,7 @@ static int init_sync(void)
 	if (register_fd(nlif_fd(STATE_SYNC(interface)), STATE(fds)) == -1)
 		return -1;
 
-	STATE_SYNC(tx_queue) = queue_create(INT_MAX, QUEUE_F_EVFD);
+	STATE_SYNC(tx_queue) = queue_create("txqueue", INT_MAX, QUEUE_F_EVFD);
 	if (STATE_SYNC(tx_queue) == NULL) {
 		dlog(LOG_ERR, "cannot create tx queue");
 		return -1;
@@ -292,6 +298,13 @@ static int init_sync(void)
 							STATE(fds)) == -1)
 		return -1;
 
+	STATE_SYNC(commit) = nfct_open(CONNTRACK, 0);
+	if (STATE_SYNC(commit) == NULL) {
+		dlog(LOG_ERR, "can't create handler to commit");
+		return -1;
+	}
+	origin_register(STATE_SYNC(commit), CTD_ORIGIN_COMMIT);
+
 	init_alarm(&STATE_SYNC(reset_cache_alarm), NULL, do_reset_cache_alarm);
 
 	/* initialization of message sequence generation */
@@ -331,6 +344,9 @@ static void kill_sync(void)
 
 	queue_destroy(STATE_SYNC(tx_queue));
 
+	origin_unregister(STATE_SYNC(commit));
+	nfct_close(STATE_SYNC(commit));
+
 	if (STATE_SYNC(sync)->kill)
 		STATE_SYNC(sync)->kill();
 }
@@ -391,28 +407,28 @@ static int local_handler_sync(int fd, int type, void *data)
 
 	switch(type) {
 	case DUMP_INTERNAL:
-		ret = fork();
+		ret = fork_process_new(CTD_PROC_ANY, 0, NULL, NULL);
 		if (ret == 0) {
 			cache_dump(STATE_SYNC(internal), fd, NFCT_O_PLAIN);
 			exit(EXIT_SUCCESS);
 		}
 		break;
 	case DUMP_EXTERNAL:
-		ret = fork();
+		ret = fork_process_new(CTD_PROC_ANY, 0, NULL, NULL);
 		if (ret == 0) {
 			cache_dump(STATE_SYNC(external), fd, NFCT_O_PLAIN);
 			exit(EXIT_SUCCESS);
 		} 
 		break;
 	case DUMP_INT_XML:
-		ret = fork();
+		ret = fork_process_new(CTD_PROC_ANY, 0, NULL, NULL);
 		if (ret == 0) {
 			cache_dump(STATE_SYNC(internal), fd, NFCT_O_XML);
 			exit(EXIT_SUCCESS);
 		}
 		break;
 	case DUMP_EXT_XML:
-		ret = fork();
+		ret = fork_process_new(CTD_PROC_ANY, 0, NULL, NULL);
 		if (ret == 0) {
 			cache_dump(STATE_SYNC(external), fd, NFCT_O_XML);
 			exit(EXIT_SUCCESS);
@@ -421,10 +437,13 @@ static int local_handler_sync(int fd, int type, void *data)
 	case COMMIT:
 		/* delete the reset alarm if any before committing */
 		del_alarm(&STATE_SYNC(reset_cache_alarm));
-		ret = fork();
+
+		/* fork new process and insert it the process list */
+		ret = fork_process_new(CTD_PROC_COMMIT, CTD_PROC_F_EXCL,
+				       NULL, NULL);
 		if (ret == 0) {
 			dlog(LOG_NOTICE, "committing external cache");
-			cache_commit(STATE_SYNC(external));
+			cache_commit(STATE_SYNC(external), STATE_SYNC(commit));
 			exit(EXIT_SUCCESS);
 		}
 		break;
@@ -475,6 +494,9 @@ static int local_handler_sync(int fd, int type, void *data)
 		multichannel_stats_extended(STATE_SYNC(channel),
 					    STATE_SYNC(interface), fd);
 		break;
+	case STATS_QUEUE:
+		queue_stats_show(fd);
+		break;
 	default:
 		if (STATE_SYNC(sync)->local)
 			ret = STATE_SYNC(sync)->local(fd, type, data);
@@ -556,7 +578,8 @@ static int resync_sync(enum nf_conntrack_msg_type type,
 	return NFCT_CB_CONTINUE;
 }
 
-static void event_new_sync(struct nf_conntrack *ct)
+static void
+event_new_sync(struct nf_conntrack *ct, int origin)
 {
 	struct cache_object *obj;
 	int id;
@@ -577,7 +600,11 @@ retry:
 			cache_object_free(obj);
 			return;
 		}
-		sync_send(obj, NET_T_STATE_NEW);
+		/* only synchronize events that have been triggered by other
+		 * processes or the kernel, but don't propagate events that
+		 * have been triggered by conntrackd itself, eg. commits. */
+		if (origin == CTD_ORIGIN_NOT_ME)
+			sync_send(obj, NET_T_STATE_NEW);
 	} else {
 		cache_del(STATE_SYNC(internal), obj);
 		cache_object_free(obj);
@@ -585,7 +612,8 @@ retry:
 	}
 }
 
-static void event_update_sync(struct nf_conntrack *ct)
+static void
+event_update_sync(struct nf_conntrack *ct, int origin)
 {
 	struct cache_object *obj;
 
@@ -593,21 +621,26 @@ static void event_update_sync(struct nf_conntrack *ct)
 	if (obj == NULL)
 		return;
 
-	sync_send(obj, NET_T_STATE_UPD);
+	if (origin == CTD_ORIGIN_NOT_ME)
+		sync_send(obj, NET_T_STATE_UPD);
 }
 
-static int event_destroy_sync(struct nf_conntrack *ct)
+static int
+event_destroy_sync(struct nf_conntrack *ct, int origin)
 {
 	struct cache_object *obj;
 	int id;
 
+	/* we don't synchronize events for objects that are not in the cache */
 	obj = cache_find(STATE_SYNC(internal), ct, &id);
 	if (obj == NULL)
 		return 0;
 
 	if (obj->status != C_OBJ_DEAD) {
 		cache_object_set_status(obj, C_OBJ_DEAD);
-		sync_send(obj, NET_T_STATE_DEL);
+		if (origin == CTD_ORIGIN_NOT_ME) {
+			sync_send(obj, NET_T_STATE_DEL);
+		}
 		cache_object_put(obj);
 	}
 	return 1;
